rec.sim = function(n, trubet=c(1,1), gam1=1, gam2=NULL, gam3=NULL, scene=1)  {
# scene = "true" scenario used to generate the data
#   1 = normal-normal homoscedastic
#   2 = normal-normal heteroscedastic
#   3 = normal-normal heteroscedastic with differential error
#   4 = normal - double exponential
#   5 = normal-modchi2
#   6 = modchi2-modchi2
#   7 = normal - skewed normal    
#   8 = modchi2 - skewed normal
#   9 = skewed normal - skewed normal
#  10 = skewed normal - modchi2

#PARAMETERS FOR MODCHI2 AND SN DISTNS
mdchmean = 0.8497415
mdchsd = 1.037242
snlambda = 50
sndel = sqrt(1/(snlambda^(-2)+1))
snlam = sndel/sqrt(1-sndel^2)
snmean = sqrt(2/pi)*sndel
snsd = sqrt(1-snmean^2)

  if (scene == 1) {
    #NORMAL-NORMAL HOMOSCEDASTIC
    x = rnorm(n,0,1)
    oddrat = exp(trubet[1] + trubet[2]*x)
    pr = oddrat/(1+oddrat)
    y = rbinom(n,1,pr)
    e = rnorm(n,0,sd=sqrt(gam1))
    w = x + e
   }

  if (scene == 2) {
    #NORMAL-NORMAL HETEROSCEDASTIC
    x = rnorm(n,0,1)
    oddrat = exp(trubet[1] + trubet[2]*x)
    pr = oddrat/(1+oddrat)
    y = rbinom(n,1,pr)
    condsig = sqrt(gam1+gam2*abs(x))
    e = rnorm(n,0,sd=condsig)
    w = x + e
   }

  if (scene == 3) {
    #NORMAL-NORMAL DIFFL ERROR
    x = rnorm(n,0,1)
    oddrat = exp(trubet[1] + trubet[2]*x)
    pr = oddrat/(1+oddrat)
    y = rbinom(n,1,pr)
    condsig = sqrt(gam1+gam2*abs(x)+gam3*y)
    e = rnorm(n,0,sd=condsig)
    w = x + e
   }

  if (scene == 4) {
    #LAPLACE (DBLE EXPL) ERROR
    x = rnorm(n,0,1)
    oddrat = exp(trubet[1] + trubet[2]*x)
    pr = oddrat/(1+oddrat)
    y = rbinom(n,1,pr)
    sig = sqrt(gam1)
    exprte = sqrt(2)/sig
    eabs = rexp(n,exprte)
    esign = 2*rbinom(n,1,0.5)-1
    e = esign*eabs
    w = x + e
  }

  if (scene == 5) {
    #NORMAL-MODCHI
    sig = sqrt(gam1)
    x = rnorm(n,0,1)
    oddrat = exp(trubet[1] + trubet[2]*x)
    pr = oddrat/(1+oddrat)
    y = rbinom(n,1,pr)
    urv = 0.9746527*runif(n)
    e = qchisq(urv,1)
    e = (e - mdchmean)/mdchsd
    e = sig*e
    w = x + e
  }

   if (scene == 6) {
     #MODCHI-MODCHI
     sig = sqrt(gam1)
     urv1 = 0.9746527*runif(n)
     x = qchisq(urv1,1)
     x = (x - mdchmean)/mdchsd
     oddrat = exp(trubet[1] + trubet[2]*x)
     pr = oddrat/(1+oddrat)
     y = rbinom(n,1,pr)    
     urv2 = 0.9746527*runif(n)
     e = qchisq(urv2,1)
     e = (e - mdchmean)/mdchsd
     e = sig*e
     w = x + e
    } 
   
   if (scene == 7) {
     #NORMAL-SN
     sig = sqrt(gam1)
     x = rnorm(n,0,1)
     oddrat = exp(trubet[1] + trubet[2]*x)
     pr = oddrat/(1+oddrat)
     y = rbinom(n,1,pr)
     sntmp = rmvnorm(n,mean=c(0,0),sigma=matrix(c(1,sndel,sndel,1),2))
     e = ifelse(sntmp[,1]>0,sntmp[,2],-sntmp[,2])
     e = (e - snmean)/snsd
     e = sig*e
     w = x + e
    }
   
   if (scene == 8) {
     #MODCHI-SN
     sig = sqrt(gam1)
     urv = 0.9746527*runif(n)
     x = qchisq(urv,1)
     x = (x - mdchmean)/mdchsd
     oddrat = exp(trubet[1] + trubet[2]*x)
     pr = oddrat/(1+oddrat)
     y = rbinom(n,1,pr)
     sntmp = rmvnorm(n,mean=c(0,0),sigma=matrix(c(1,sndel,sndel,1),2))
     e = ifelse(sntmp[,1]>0,sntmp[,2],-sntmp[,2])
     e = (e - snmean)/snsd
     e = sig*e
     w = x + e
    }
     
   if (scene == 9) {
     #SN-SN
     sig = sqrt(gam1)
     sntmp = rmvnorm(n,mean=c(0,0),sigma=matrix(c(1,sndel,sndel,1),2))
     x = ifelse(sntmp[,1]>0,sntmp[,2],-sntmp[,2])
     x = (x - snmean)/snsd
     oddrat = exp(trubet[1] + trubet[2]*x)
     pr = oddrat/(1+oddrat)
     y = rbinom(n,1,pr)
     sntmp = rmvnorm(n,mean=c(0,0),sigma=matrix(c(1,sndel,sndel,1),2))
     e = ifelse(sntmp[,1]>0,sntmp[,2],-sntmp[,2])
     e = (e - snmean)/snsd
     e = sig*e
     w = x + e
    }    
     
   if (scene == 10) {
     #SN-MODCHI
     sig = sqrt(gam1)
     sntmp = rmvnorm(n,mean=c(0,0),sigma=matrix(c(1,sndel,sndel,1),2))
     x = ifelse(sntmp[,1]>0,sntmp[,2],-sntmp[,2])
     x = (x - snmean)/snsd
     oddrat = exp(trubet[1] + trubet[2]*x)
     pr = oddrat/(1+oddrat)
     y = rbinom(n,1,pr)
     urv = 0.9746527*runif(n)
     e = qchisq(urv,1)
     e = (e - mdchmean)/mdchsd
     e = sig*e
     w = x + e
    }       
     
return(list(y=y, x=x, w=w))
}