
library(mvtnorm)
library(mc2d)
library(msm)
library(coda)

source("SimScene.R")
source("main.R")
source("updateBeta.R")
source("updateTau.R")
source("updateX.R")
source("updateOMuSigBC.R")
source("rjmcmcSM.R")
source("rjmcmcBD.R")
source("logLik.R")


clusize = function(x, k) {
	n.clu=rep(NA, k)
	for(i in 1:k)  n.clu[i]= length(which(x==i))
	return(n.clu)
}

		
logddir = function(x, dirpar) {
        logD = sum(lgamma(dirpar)) - lgamma(sum(dirpar))
        s = sum((dirpar - 1) * log(x))
        return (sum(s) - logD)
}


modchi = function(x, gam, log=TRUE) {
	mdchmean = 0.8497415; mdchsd = 1.037242; sig=mdchsd/sqrt(gam)
	x.star = x*sig+mdchmean
	if(log==TRUE) {
		return (  log(sig) + dchisq(x.star, 1, log=T) - pchisq(5,1, log=T) ) 
	} else {
		return ( sig * dchisq(x.star, 1, log=F)/pchisq(5,1,log=F) ) 
	}
}
	

dblexp = function(x, gam, log=TRUE) {
	lambda = sqrt(2/gam)
	if(log==TRUE) {
		return( log(lambda/2) - lambda*abs(x) )
	} else {
		return( lambda/2 * exp(-lambda*abs(x)) )
	}
}

#PARAMETERS FOR MODCHI2 AND SN DISTNS
mdchmean = 0.8497415
mdchsd = 1.037242
snlambda = 50
sndel = sqrt(1/(snlambda^(-2)+1))
snlam = sndel/sqrt(1-sndel^2)
snmean = sqrt(2/pi)*sndel
snsd = sqrt(1-snmean^2)



B=10 	# number of simulation replications 
Beta0.mean = Beta0.sd = Beta1.mean = Beta1.sd = rep(NA, B)
Beta0.ci = Beta0.hpd = Beta1.ci = Beta1.hpd = matrix(NA, B, 2)
cov.beta0.ci = cov.beta0.hpd = cov.beta1.ci = cov.beta1.hpd = rep(NA, B)

run=1
seed0 = seq(10*(run-1)+1,10*run)+11817

for(b in 1:B)  {
set.seed(seed0[b])

# simulate data
nmain=200; nval=70; n=nmain+nval; trubet=c(1,1); gam1=0.7; gam2=0.35; scene=2
simdat = rec.sim(nmain, nval, trubet, gam1, scene)  
sel.val = sample(1:n, nval)
y.valtrue=simdat$y[sel.val]; y.main=simdat$y[-sel.val];
x.val = simdat$x[sel.val]; x.true=simdat$x[-sel.val];
w.val=simdat$w[sel.val]; w.main=simdat$w[-sel.val]

# set-up initial values and hyperparameters 
fit.naive = summary(glm(y.main ~ w.main, family="binomial"))
beta.tilde = fit.naive$coeff[,1]
beta.var = fit.naive$coeff[,2]^2

eta = beta.tilde[1]+beta.tilde[2]*x.val; py = exp(eta)/(1+exp(eta))
y.valsamp = rbinom(nval,1,py)

e.val = w.val - x.val
gam.tilde = var(e.val)
if (scene==4) {
	sig = sqrt(gam.tilde); exprte = sqrt(2)/sig
	eabs = rexp(nmain, exprte); 
	esign = 2*rbinom(nmain, 1, 0.5)-1
	e.main = esign*eabs 
} else if (scene %in% c(5,6,10)) {
	urv = 0.9746527*runif(nmain)
	e.main = qchisq(urv, 1)
	e.main = (e.main - mdchmean)/mdchsd
	e.main = sqrt(gam.tilde)*e.main
} else {
	e.main = rnorm(nmain, 0, sqrt(gam.tilde)) 
} 
x.main = w.main - e.main

R = max(x.main)-min(x.main)
kappa = 1/R^2
h.sig = 10/R^2
xi = min(x.main)+R/2

h.beta=diag(100*beta.var); a.tau=0.01; b.tau=0.01; a.sig=2; g.sig=0.2; delta=1; Kmax=30

# initial values
bet=beta.tilde; 
if (scene==2) {
	fit.ex = summary(lm(e.val ~ abs(x.val)))
	tau= 1/fit.ex$coeff[,1]
} else if(scene==3) {
	fit.exy = summary(lm(e.val ~ abs(x.val) + y.valsamp))
	tau = 1/fit.exy$coeff[,1]
} else { tau = 1/gam.tilde } 
	

z = rep(1,n); mu=rnorm(1, xi, sqrt(1/kappa));  omega=1
k=length(unique(z)); nk = n
b.sig=rgamma(1, g.sig, h.sig); sigma2 = 1/rgamma(1, a.sig, b.sig)

T = 100000; nburn=60000; thin=1	# number of MCMC iterations and burn-in
X=matrix(NA, T, nmain); Y=matrix(NA, T, nval); Z=matrix(NA, T, n)
Beta = matrix(NA, T, 2); Tau = matrix(NA, T, length(tau)); Bsig=rep(NA, T); LogPost=rep(NA, T)
K=matrix(NA, T,3); Nk = Omega = Mu = Sigma2 = vector("list", T)

priorK=2;			# 1: uniform   2: Poisson
lpois=3;			# parameter of trunc-Poisson prior
AssumeScene=scene;	# 1-10 assumed scenario (may be different than true scenario)

Result = main.fun(y.valsamp, w.val, x.val, y.main, w.main, x.main, beta.tilde, h.beta, a.tau, b.tau, xi, kappa, a.sig, g.sig, h.sig, delta, Kmax, bet, tau, omega, z, nk, k, b.sig, priorK, lpois, AssumeScene)  

# save(Result,y.valtrue, y.main, x.val, x.true, w.val, w.main, beta.tilde, h.beta, a.tau, b.tau, xi, kappa, a.sig, g.sig, h.sig, delta, Kmax, file= "Sim_Result.RData")

X=Result$X; Y=Result$Y; Z=Result$Z
Beta=Result$Beta; Tau=Result$Tau; Bsig=Result$Bsig
K=Result$K; Nk=Result$Nk; Omega=Result$Omega; Mu=Result$Mu; Sigma2=Result$Sigma2

Beta0.post = Beta[seq(nburn,T,thin),1]; Beta1.post = Beta[seq(nburn,T,thin),2]

Beta0.mean[b] = mean(Beta0.post); Beta0.sd[b] = sd(Beta0.post)
Beta0.ci[b,] = as.vector(quantile(Beta0.post, prob=c(0.025, 0.975)))
cov.beta0.ci[b] = as.numeric(trubet[1] >= Beta0.ci[b,1] & trubet[1] <= Beta0.ci[b,2])
Beta0.hpd[b,] = as.vector(HPDinterval(as.mcmc(Beta0.post), prob=0.95))
cov.beta0.hpd[b] = as.numeric(trubet[1] >= Beta0.hpd[b,1] & trubet[1] <= Beta0.hpd[b,2])

Beta1.mean[b] = mean(Beta1.post); Beta1.sd[b] = sd(Beta1.post)
Beta1.ci[b,] = as.vector(quantile(Beta1.post, prob=c(0.025, 0.975))); 
cov.beta1.ci[b] = as.numeric(trubet[2] >= Beta1.ci[b,1] & trubet[2] <= Beta1.ci[b,2])
Beta1.hpd[b,] = as.vector(HPDinterval(as.mcmc(Beta1.post), prob=0.95))
cov.beta1.hpd[b] = as.numeric(trubet[2] >= Beta1.hpd[b,1] & trubet[2] <= Beta1.hpd[b,2])

}

save(Beta0.mean, Beta0.sd, Beta0.ci, cov.beta0.ci, Beta0.hpd, cov.beta0.hpd, Beta1.mean, Beta1.sd, Beta1.ci, cov.beta1.ci, Beta1.hpd, cov.beta1.hpd, file="ScenarioA2_gam07_rep1.RData")


