
library(mvtnorm)
library(mc2d)
library(msm)
library(coda)

source("Sim_sceneD.R")
source("main_sceneD.R")
source("updateBeta_sceneD.R")
source("updateX_sceneD.R")
source("updateOMuSigBC.R")
source("rjmcmcSM_sceneD.R")
source("rjmcmcBD_sceneD.R")
source("logLik_sceneD.R")


clusize = function(x, k) {
	n.clu=rep(NA, k)
	for(i in 1:k)  n.clu[i]= length(which(x==i))
	return(n.clu)
}

		
logddir = function(x, dirpar) {
        logD = sum(lgamma(dirpar)) - lgamma(sum(dirpar))
        s = sum((dirpar - 1) * log(x))
        return (sum(s) - logD)
}


modchi = function(x, gam, log=TRUE) {
	mdchmean = 0.8497415; mdchsd = 1.037242
	x.star = x*mdchsd/sqrt(gam)+mdchmean
	if(log==TRUE) {
		return (  log(mdchsd/sqrt(gam)) + dchisq(x.star, 1, log=T) - pchisq(5,1, log=T) ) 
	} else {
		return ( mdchsd/sqrt(gam) * dchisq(x.star, 1, log=F)/pchisq(5,1,log=F) ) 
	}
}
	
#PARAMETERS FOR MODCHI2
mdchmean = 0.8497415
mdchsd = 1.037242


B=1 	# number of simulation replications 
Beta0.mean = Beta1.mean = Beta2.mean = Beta3.mean =rep(NA, B)
Beta0.sd = Beta1.sd = Beta2.sd = Beta3.sd = rep(NA, B)
Beta0.ci = Beta1.ci = Beta2.ci = Beta3.ci = matrix(NA, B, 2)
Beta0.hpd = Beta1.hpd = Beta2.hpd = Beta3.hpd = matrix(NA, B, 2)
cov.beta0.ci = cov.beta1.ci = cov.beta2.ci = cov.beta3.ci = rep(NA, B)
cov.beta0.hpd = cov.beta1.hpd = cov.beta2.hpd = cov.beta3.hpd = rep(NA, B)

run=1
seed0 = seq(5*(run-1)+1,5*run)+11817

for(b in 1:B)  {
set.seed(seed0[b])

# simulate data
n=500; trubet=rep(1,4); gam1=1; gam2=NULL; scene="D1"
simdat = rec.simD(n, trubet, gam1=gam1, gam2=gam2, scene=scene)  
y=simdat$y; x1.true=simdat$x[,1]; x2.true=simdat$x[,2]; x3=simdat$x[,3]
w1=simdat$w[,1]; w2=simdat$w[,2]

# set-up initial values and hyperparameters 
fit.naive = summary(glm(y ~ cbind(w1,w2,x3), family="binomial"))
beta.tilde = fit.naive$coeff[,1]
beta.var = fit.naive$coeff[,2]^2

if (scene=="D2") {
	sig = sqrt(gam1+gam2*(abs(w1)+abs(w2)+abs(x3)))
	e1 = rnorm(n, 0, sd=sig); e2 = rnorm(n, 0, sd=sig)
} else if (scene=="D3")  {
      sig=sqrt(gam1); 
      urv.e1 = 0.9746527*runif(n); urv.e2= 0.9746527*runif(n)
      e1 = qchisq(urv.e1,1); e2=qchisq(urv.e2,1)
      e1 = (e1-mdchmean)/mdchsd; e2 = (e2-mdchmean)/mdchsd
      e1 = sig*e1; e2 = sig*e2
} else {
	e1 = rnorm(n, 0, sqrt(gam1)); e2 = rnorm(n, 0, sqrt(gam1)) 
} 
x1 = w1+e1; x2 = w2+e2

R1 = max(x1)-min(x1); R2 = max(x2)-min(x2)
kappa1 = 1/R1^2; kappa2 = 1/R2^2
h1.sig = 10/R1^2; h2.sig = 10/R2^2
xi1 = min(x1)+R1/2; xi2 = min(x2)+R2/2

h.beta=diag(100*beta.var); a.sig=2; g.sig=0.2; delta=1; Kmax=30

# initial values

bet=as.vector(beta.tilde); 

z1=z2=rep(1,n); omega1=omega2=1
mu1=rnorm(1, xi1, sqrt(1/kappa1)); mu2=rnorm(1, xi2, sqrt(1/kappa2))
k1=length(unique(z1)); k2 = length(unique(z2)); nk1=nk2=n
b1.sig=rgamma(1, g.sig, h1.sig); b2.sig=rgamma(1, g.sig, h2.sig)
sigma2.1 = 1/rgamma(1, a.sig, b1.sig); sigma2.2 = 1/rgamma(1, a.sig, b2.sig)

T = 100000; nburn=60000; thin=1	# number of MCMC iterations and burn-in
X1=X2=matrix(NA, T, n); Z1=Z2=matrix(NA, T, n); K1=K2=matrix(NA, T,3)
Beta = matrix(NA, T, 4); Bsig1=Bsig2=rep(NA, T); LogPost=rep(NA, T)
Nk1=Nk2=Omega1=Omega2=Mu1=Mu2=Sigma2.1=Sigma2.2=vector("list", T)

priorK=2;			# 1: uniform   2: Poisson
lpois=3;			# parameter of trunc-Poisson prior
AssumeScene=scene;	# D1-D3 assumed scenario (may be different than true scenario)

Result = main.fun(y, w1, w2, x3, x1, x2, beta.tilde, h.beta, xi1, xi2, gam1, gam2, kappa1, kappa2, a.sig, g.sig, h1.sig, h2.sig, delta, Kmax, bet, omega1, omega2, z1, z2, nk1, nk2, k1, k2, b1.sig, b2.sig, priorK, lpois, AssumeScene)  

# save(Result,y, x1.true, x2.true, x3, w1, w2, beta.tilde, h.beta, xi1, xi2, kappa1, kappa2, a.sig, g.sig, h1.sig, h2.sig, delta, Kmax, file= "SimD_Result.RData")

Beta = Result$Beta
# X1=Result$X1; X2=Result$X2; Z1=Result$Z1; Z2=Result$Z2
# Bsig1=Result$Bsig1; Bsig2=Result$Bsig2
# K1=Result$K1; K2=Result$K2; Nk1=Result$Nk1; Nk2=Result$Nk2; 
# Omega1=Result$Omega1; Omega2=Result$Omega2
# Mu1=Result$Mu1; Mu2=Result$Mu2; Sigma2.1=Result$Sigma2.1; Sigma2.2=Result$Sigma2.2

Beta0.post = Beta[seq(nburn,T,thin),1]; Beta1.post = Beta[seq(nburn,T,thin),2]
Beta2.post = Beta[seq(nburn,T,thin),3]; Beta3.post = Beta[seq(nburn,T,thin),4]

Beta0.mean[b] = mean(Beta0.post); Beta0.sd[b] = sd(Beta0.post)
Beta0.ci[b,] = as.vector(quantile(Beta0.post, prob=c(0.025, 0.975)))
cov.beta0.ci[b] = as.numeric(trubet[1] >= Beta0.ci[b,1] & trubet[1] <= Beta0.ci[b,2])
Beta0.hpd[b,] = as.vector(HPDinterval(as.mcmc(Beta0.post), prob=0.95))
cov.beta0.hpd[b] = as.numeric(trubet[1] >= Beta0.hpd[b,1] & trubet[1] <= Beta0.hpd[b,2])

Beta1.mean[b] = mean(Beta1.post); Beta1.sd[b] = sd(Beta1.post)
Beta1.ci[b,] = as.vector(quantile(Beta1.post, prob=c(0.025, 0.975))); 
cov.beta1.ci[b] = as.numeric(trubet[2] >= Beta1.ci[b,1] & trubet[2] <= Beta1.ci[b,2])
Beta1.hpd[b,] = as.vector(HPDinterval(as.mcmc(Beta1.post), prob=0.95))
cov.beta1.hpd[b] = as.numeric(trubet[2] >= Beta1.hpd[b,1] & trubet[2] <= Beta1.hpd[b,2])

Beta2.mean[b] = mean(Beta2.post); Beta2.sd[b] = sd(Beta2.post)
Beta2.ci[b,] = as.vector(quantile(Beta2.post, prob=c(0.025, 0.975))); 
cov.beta2.ci[b] = as.numeric(trubet[3] >= Beta2.ci[b,1] & trubet[3] <= Beta2.ci[b,2])
Beta2.hpd[b,] = as.vector(HPDinterval(as.mcmc(Beta2.post), prob=0.95))
cov.beta2.hpd[b] = as.numeric(trubet[3] >= Beta2.hpd[b,1] & trubet[3] <= Beta2.hpd[b,2])

Beta3.mean[b] = mean(Beta3.post); Beta3.sd[b] = sd(Beta3.post)
Beta3.ci[b,] = as.vector(quantile(Beta3.post, prob=c(0.025, 0.975))); 
cov.beta3.ci[b] = as.numeric(trubet[4] >= Beta3.ci[b,1] & trubet[4] <= Beta3.ci[b,2])
Beta3.hpd[b,] = as.vector(HPDinterval(as.mcmc(Beta3.post), prob=0.95))
cov.beta3.hpd[b] = as.numeric(trubet[4] >= Beta3.hpd[b,1] & trubet[4] <= Beta3.hpd[b,2])
 
}

save(Beta0.mean, Beta0.sd, Beta0.ci, cov.beta0.ci, Beta0.hpd, cov.beta0.hpd, Beta1.mean, Beta1.sd, Beta1.ci, cov.beta1.ci, Beta1.hpd, cov.beta1.hpd, Beta2.mean, Beta2.sd, Beta2.ci, cov.beta2.ci, Beta2.hpd, cov.beta2.hpd, Beta3.mean, Beta3.sd, Beta3.ci, cov.beta3.ci, Beta3.hpd, cov.beta3.hpd, file="ScenarioD1v2_rep51.RData")

