options(width=200) #to get long line
options(digits=4) # for printing 4 digits after the decimal point

dyn.load("hwdr.so")
boots = 100

hw = function(n,y,w1,w2) {
 
  # naive estimate as starting value
  wavg = (w1+w2)/2
  naive = glm(y~wavg,family=binomial(link='logit'))
  beta0 = naive$coef
 
  # setups for hw method
 
  # npred = number of predictor variables
  # nrep[i,j] = number of replicate person i has on covariate j-1 (intecept = covariate 0)
  # maxrep = max number of replicates
  # derr = error tolerance
  # succ = code for successful estimation
 
  nr0 = rep(1,n)
  nr1 = rep(2,n)
  # assumes two replicates of w for each person
  nrep = cbind(nr0,nr1)
  w = array(rep(1,4*n),dim=c(n,2,2))
  for (i in 1:n) {
    w[i,2,1] = w1[i]
    w[i,2,2] = w2[i]
   } 
  npred = 1
  maxrep = 2
  derr = 1e-3
  ndwork = 5*npred + 15
  beta = rep(0,npred+1)
  succ = c(1)
  
  storage.mode(y) = "integer"
  storage.mode(w) = "double"
  storage.mode(nrep) = "integer"
  storage.mode(n) = "integer"
  storage.mode(npred) = "integer"
  storage.mode(maxrep) = "integer"
  storage.mode(beta0) = "double"
  storage.mode(derr) = "double"
  storage.mode(ndwork) = "integer"
  storage.mode(beta) = "double"
  storage.mode(succ) = "integer"
 
  # run hw method
  hwres = .Fortran('hwdr',y,w,nrep,n,npred,maxrep,
    beta0,derr,ndwork,beta=beta,succ=succ)
  betvec = hwres$beta
  b0temp <- ifelse(abs(betvec[1])<=5,betvec[1],NA)
  b1temp <- ifelse(abs(betvec[2])<=5,betvec[2],NA)
  beta_est = c(b0temp,b1temp)
  succ = hwres$succ
  results = list(beta_est,succ)
  names(results) = c("beta","succ")
  return(results)
  
 }

#####################################################################
#2-1 a
n = 200 
nsamples = 1000
ea = 1.0
seed0 = seq(1,nsamples)+11817
trubet = c(1,1)
bet1 = NULL
bet0 = NULL
bbet1 = NULL
bbet0 = NULL
cicov0 = NULL
cicov1 = NULL
bsds0 = NULL
bsds1 = NULL
nsucc = 0
 
for (isam in 1:nsamples) {
  #LAPLACE (DBLE EXPL) ERROR
  set.seed(seed0[isam])
  x = rnorm(n,0,1)
  oddrat = exp(trubet[1] + trubet[2]*x)
  pr = oddrat/(1+oddrat)
  y = rbinom(n,1,pr)
  sig = sqrt(ea)
  exprte = sqrt(2)/sig
  eabs = rexp(n,exprte)
  esign = 2*rbinom(n,1,0.5)-1
  e1 = esign*eabs
  w1 = x + e1
  eabs = rexp(n,exprte)
  esign = 2*rbinom(n,1,0.5)-1
  e2 = esign*eabs
  w2 = x + e2
  hwres = hw(n,y,w1,w2)
  sucval = hwres$succ
  if (sucval==1) {
    nsucc = nsucc + 1
    bet0val = (hwres$beta)[1]
    bet0 = c(bet0,bet0val)
    bet1val = (hwres$beta)[2]
    bet1 = c(bet1,bet1val)
   } 
  #bootstrap for SIMEX
  btsest = array(rep(0,6*samples),dim=c(samples,2,3))
    for (bb in 1:boots) {
    bindx<-sample(n,replace=T)
    by = y[bindx]; bw1 = w1[bindx]; bw2 = w2[bindx]
    bhwres = hw(n,by,bw1,bw2)
    sucval = bhwres$succ  
    if (sucval==1) {   
      bbet0val = (hwres$beta)[1]
      bbet0 = c(bbet0,bbet0val)
      bbet1val = (bhwres$beta)[2]
      bbet1 = c(bbet1,bbet1val)
    }
  }#end bootstrap
  bsds0[i] = sd(bbet0,na.rm=T)
  bsds1[i] = sd(bbet1,na.rm=T)
  diff = abs(bet0val-trubet[1]) - 1.96*bsds0[i]
  cicov0[i] = (diff <= 0)
  diff = abs(bet1val-trubet[2]) - 1.96*bsds1[i]
  cicov1[i] = (diff <= 0)
 }

pctsucc = nsucc/nsamples
print(summary(bet0))
meanb0 = mean(bet0,na.rm=TRUE)
medb0 = median(bet0,na.rm=TRUE)
sdb0 = sd(bet0,na.rm=TRUE)
sumstat0 = summary(bet0)
adjiqr0 = (sumstat0[5]-sumstat0[2])/1.349
ci0 = mean(cicov0,na.rm=T)
estsd = mean(bsds0,na.rm=T)  
results = cbind(ea,n,nsamples,pctsucc,meanb0,medb0,sdb0,adjiqr0,ci0,estsd)
print(results)

print(summary(bet1))
meanb1 = mean(bet1,na.rm=TRUE)
medb1 = median(bet1,na.rm=TRUE)
sdb1 = sd(bet1,na.rm=TRUE)
sumstat1 = summary(bet1)
adjiqr1 = (sumstat1[5]-sumstat1[2])/1.349
ci1 = mean(cicov1,na.rm=T)
estsd = mean(bsds1,na.rm=T)
results = cbind(ea,n,nsamples,pctsucc,meanb1,medb1,sdb1,adjiqr1,ci1,estsd)
print(results)
     

#####################################################################
#2-2
n = 200 
nsamples = 1000
ea = 0.5
seed0 = seq(1,nsamples)+11817
trubet = c(1,1)
bet1 = NULL
bet0 = NULL
bbet1 = NULL
bbet0 = NULL
cicov0 = NULL
cicov1 = NULL
bsds0 = NULL
bsds1 = NULL
nsucc = 0
mdchmean = 0.8497415
mdchsd = 1.037242
 
for (isam in 1:nsamples) {
  #NORMAL-MODCHI
  set.seed(seed0[isam])
  x = rnorm(n,0,1)
  oddrat = exp(trubet[1] + trubet[2]*x)
  pr = oddrat/(1+oddrat)
  y = rbinom(n,1,pr)
  urv = 0.9746527*runif(n)
  e = qchisq(urv,1)
  e = (e - mdchmean)/mdchsd
  sig = sqrt(ea)
  e1 = sig*e
  w1 = x + e1
  urv = 0.9746527*runif(n)
  e = qchisq(urv,1)
  e = (e - mdchmean)/mdchsd
  e2 = sig*e
  w2 = x + e2
  hwres = hw(n,y,w1,w2)
  sucval = hwres$succ
  if (sucval==1) {
    nsucc = nsucc + 1
    bet0val = (hwres$beta)[1]
    bet0 = c(bet0,bet0val)
    bet1val = (hwres$beta)[2]
    bet1 = c(bet1,bet1val)
   } 
  #bootstrap for SIMEX
  btsest = array(rep(0,6*samples),dim=c(samples,2,3))
    for (bb in 1:boots) {
    bindx<-sample(n,replace=T)
    by = y[bindx]; bw1 = w1[bindx]; bw2 = w2[bindx]
    bhwres = hw(n,by,bw1,bw2)
    sucval = bhwres$succ  
    if (sucval==1) {   
      bbet0val = (hwres$beta)[1]
      bbet0 = c(bbet0,bbet0val)
      bbet1val = (bhwres$beta)[2]
      bbet1 = c(bbet1,bbet1val)
    }
  }#end bootstrap
  bsds0[i] = sd(bbet0,na.rm=T)
  bsds1[i] = sd(bbet1,na.rm=T)
  diff = abs(bet0val-trubet[1]) - 1.96*bsds0[i]
  cicov0[i] = (diff <= 0)
  diff = abs(bet1val-trubet[2]) - 1.96*bsds1[i]
  cicov1[i] = (diff <= 0)
 }

pctsucc = nsucc/nsamples
print(summary(bet0))
meanb0 = mean(bet0,na.rm=TRUE)
medb0 = median(bet0,na.rm=TRUE)
sdb0 = sd(bet0,na.rm=TRUE)
sumstat0 = summary(bet0)
adjiqr0 = (sumstat0[5]-sumstat0[2])/1.349
ci0 = mean(cicov0,na.rm=T)
estsd = mean(bsds0,na.rm=T)  
results = cbind(ea,n,nsamples,pctsucc,meanb0,medb0,sdb0,adjiqr0,ci0,estsd)
print(results)

print(summary(bet1))
meanb1 = mean(bet1,na.rm=TRUE)
medb1 = median(bet1,na.rm=TRUE)
sdb1 = sd(bet1,na.rm=TRUE)
sumstat1 = summary(bet1)
adjiqr1 = (sumstat1[5]-sumstat1[2])/1.349
ci1 = mean(cicov1,na.rm=T)
estsd = mean(bsds1,na.rm=T)
results = cbind(ea,n,nsamples,pctsucc,meanb1,medb1,sdb1,adjiqr1,ci1,estsd)
print(results)    

#####################################################################
#2-3
n = 200 
nsamples = 1000
ea = 1.0
seed0 = seq(1,nsamples)+11817
trubet = c(1,1)
bet1 = NULL
bet0 = NULL
bbet1 = NULL
bbet0 = NULL
cicov0 = NULL
cicov1 = NULL
bsds0 = NULL
bsds1 = NULL
nsucc = 0
mdchmean = 0.8497415
mdchsd = 1.037242
 
for (isam in 1:nsamples) {
  #NORMAL-MODCHI
  set.seed(seed0[isam])
  x = rnorm(n,0,1)
  oddrat = exp(trubet[1] + trubet[2]*x)
  pr = oddrat/(1+oddrat)
  y = rbinom(n,1,pr)
  urv = 0.9746527*runif(n)
  e = qchisq(urv,1)
  e = (e - mdchmean)/mdchsd
  sig = sqrt(ea)
  e1 = sig*e
  w1 = x + e1
  urv = 0.9746527*runif(n)
  e = qchisq(urv,1)
  e = (e - mdchmean)/mdchsd
  e2 = sig*e
  w2 = x + e2
  hwres = hw(n,y,w1,w2)
  sucval = hwres$succ
  if (sucval==1) {
    nsucc = nsucc + 1
    bet0val = (hwres$beta)[1]
    bet0 = c(bet0,bet0val)
    bet1val = (hwres$beta)[2]
    bet1 = c(bet1,bet1val)
   } 
  #bootstrap for SIMEX
  btsest = array(rep(0,6*samples),dim=c(samples,2,3))
    for (bb in 1:boots) {
    bindx<-sample(n,replace=T)
    by = y[bindx]; bw1 = w1[bindx]; bw2 = w2[bindx]
    bhwres = hw(n,by,bw1,bw2)
    sucval = bhwres$succ  
    if (sucval==1) {   
      bbet0val = (hwres$beta)[1]
      bbet0 = c(bbet0,bbet0val)
      bbet1val = (bhwres$beta)[2]
      bbet1 = c(bbet1,bbet1val)
    }
  }#end bootstrap
  bsds0[i] = sd(bbet0,na.rm=T)
  bsds1[i] = sd(bbet1,na.rm=T)
  diff = abs(bet0val-trubet[1]) - 1.96*bsds0[i]
  cicov0[i] = (diff <= 0)
  diff = abs(bet1val-trubet[2]) - 1.96*bsds1[i]
  cicov1[i] = (diff <= 0)
 }

pctsucc = nsucc/nsamples
print(summary(bet0))
meanb0 = mean(bet0,na.rm=TRUE)
medb0 = median(bet0,na.rm=TRUE)
sdb0 = sd(bet0,na.rm=TRUE)
sumstat0 = summary(bet0)
adjiqr0 = (sumstat0[5]-sumstat0[2])/1.349
ci0 = mean(cicov0,na.rm=T)
estsd = mean(bsds0,na.rm=T)  
results = cbind(ea,n,nsamples,pctsucc,meanb0,medb0,sdb0,adjiqr0,ci0,estsd)
print(results)

print(summary(bet1))
meanb1 = mean(bet1,na.rm=TRUE)
medb1 = median(bet1,na.rm=TRUE)
sdb1 = sd(bet1,na.rm=TRUE)
sumstat1 = summary(bet1)
adjiqr1 = (sumstat1[5]-sumstat1[2])/1.349
ci1 = mean(cicov1,na.rm=T)
estsd = mean(bsds1,na.rm=T)
results = cbind(ea,n,nsamples,pctsucc,meanb1,medb1,sdb1,adjiqr1,ci1,estsd)
print(results)
     
#####################################################################
#2-4
n = 200 
nsamples = 1000
ea = 0.5
seed0 = seq(1,nsamples)+11817
trubet = c(1,1)
bet1 = NULL
bet0 = NULL
bbet1 = NULL
bbet0 = NULL
cicov0 = NULL
cicov1 = NULL
bsds0 = NULL
bsds1 = NULL
nsucc = 0
mdchmean = 0.8497415
mdchsd = 1.037242
 
for (isam in 1:nsamples) {
  #MODCHI-MODCHI
  set.seed(seed0[isam])
  urv = 0.9746527*runif(n)
  x = qchisq(urv,1)
  x = (x - mdchmean)/mdchsd
  oddrat = exp(trubet[1] + trubet[2]*x)
  pr = oddrat/(1+oddrat)
  y = rbinom(n,1,pr)
  urv = 0.9746527*runif(n)
  e = qchisq(urv,1)
  e = (e - mdchmean)/mdchsd
  sig = sqrt(ea)
  e1 = sig*e
  w1 = x + e1
  urv = 0.9746527*runif(n)
  e = qchisq(urv,1)
  e = (e - mdchmean)/mdchsd
  e2 = sig*e
  w2 = x + e2
  hwres = hw(n,y,w1,w2)
  sucval = hwres$succ
  if (sucval==1) {
    nsucc = nsucc + 1
    bet0val = (hwres$beta)[1]
    bet0 = c(bet0,bet0val)
    bet1val = (hwres$beta)[2]
    bet1 = c(bet1,bet1val)
   } 
  #bootstrap for SIMEX
  btsest = array(rep(0,6*samples),dim=c(samples,2,3))
    for (bb in 1:boots) {
    bindx<-sample(n,replace=T)
    by = y[bindx]; bw1 = w1[bindx]; bw2 = w2[bindx]
    bhwres = hw(n,by,bw1,bw2)
    sucval = bhwres$succ  
    if (sucval==1) {   
      bbet0val = (hwres$beta)[1]
      bbet0 = c(bbet0,bbet0val)
      bbet1val = (bhwres$beta)[2]
      bbet1 = c(bbet1,bbet1val)
    }
  }#end bootstrap
  bsds0[i] = sd(bbet0,na.rm=T)
  bsds1[i] = sd(bbet1,na.rm=T)
  diff = abs(bet0val-trubet[1]) - 1.96*bsds0[i]
  cicov0[i] = (diff <= 0)
  diff = abs(bet1val-trubet[2]) - 1.96*bsds1[i]
  cicov1[i] = (diff <= 0)
 }

pctsucc = nsucc/nsamples
print(summary(bet0))
meanb0 = mean(bet0,na.rm=TRUE)
medb0 = median(bet0,na.rm=TRUE)
sdb0 = sd(bet0,na.rm=TRUE)
sumstat0 = summary(bet0)
adjiqr0 = (sumstat0[5]-sumstat0[2])/1.349
ci0 = mean(cicov0,na.rm=T)
estsd = mean(bsds0,na.rm=T)  
results = cbind(ea,n,nsamples,pctsucc,meanb0,medb0,sdb0,adjiqr0,ci0,estsd)
print(results)

print(summary(bet1))
meanb1 = mean(bet1,na.rm=TRUE)
medb1 = median(bet1,na.rm=TRUE)
sdb1 = sd(bet1,na.rm=TRUE)
sumstat1 = summary(bet1)
adjiqr1 = (sumstat1[5]-sumstat1[2])/1.349
ci1 = mean(cicov1,na.rm=T)
estsd = mean(bsds1,na.rm=T)
results = cbind(ea,n,nsamples,pctsucc,meanb1,medb1,sdb1,adjiqr1,ci1,estsd)
print(results)

#####################################################################
#2-5
n = 200 
nsamples = 1000
ea = 1.0
seed0 = seq(1,nsamples)+11817
trubet = c(1,1)
bet1 = NULL
bet0 = NULL
bbet1 = NULL
bbet0 = NULL
cicov0 = NULL
cicov1 = NULL
bsds0 = NULL
bsds1 = NULL
nsucc = 0
mdchmean = 0.8497415
mdchsd = 1.037242
 
for (isam in 1:nsamples) {
  #MODCHI-MODCHI
  set.seed(seed0[isam])
  urv = 0.9746527*runif(n)
  x = qchisq(urv,1)
  x = (x - mdchmean)/mdchsd
  oddrat = exp(trubet[1] + trubet[2]*x)
  pr = oddrat/(1+oddrat)
  y = rbinom(n,1,pr)
  urv = 0.9746527*runif(n)
  e = qchisq(urv,1)
  e = (e - mdchmean)/mdchsd
  sig = sqrt(ea)
  e1 = sig*e
  w1 = x + e1
  urv = 0.9746527*runif(n)
  e = qchisq(urv,1)
  e = (e - mdchmean)/mdchsd
  e2 = sig*e
  w2 = x + e2
  hwres = hw(n,y,w1,w2)
  sucval = hwres$succ
  if (sucval==1) {
    nsucc = nsucc + 1
    bet0val = (hwres$beta)[1]
    bet0 = c(bet0,bet0val)
    bet1val = (hwres$beta)[2]
    bet1 = c(bet1,bet1val)
   } 
  #bootstrap for SIMEX
  btsest = array(rep(0,6*samples),dim=c(samples,2,3))
    for (bb in 1:boots) {
    bindx<-sample(n,replace=T)
    by = y[bindx]; bw1 = w1[bindx]; bw2 = w2[bindx]
    bhwres = hw(n,by,bw1,bw2)
    sucval = bhwres$succ  
    if (sucval==1) {   
      bbet0val = (hwres$beta)[1]
      bbet0 = c(bbet0,bbet0val)
      bbet1val = (bhwres$beta)[2]
      bbet1 = c(bbet1,bbet1val)
    }
  }#end bootstrap
  bsds0[i] = sd(bbet0,na.rm=T)
  bsds1[i] = sd(bbet1,na.rm=T)
  diff = abs(bet0val-trubet[1]) - 1.96*bsds0[i]
  cicov0[i] = (diff <= 0)
  diff = abs(bet1val-trubet[2]) - 1.96*bsds1[i]
  cicov1[i] = (diff <= 0)
 }

pctsucc = nsucc/nsamples
print(summary(bet0))
meanb0 = mean(bet0,na.rm=TRUE)
medb0 = median(bet0,na.rm=TRUE)
sdb0 = sd(bet0,na.rm=TRUE)
sumstat0 = summary(bet0)
adjiqr0 = (sumstat0[5]-sumstat0[2])/1.349
ci0 = mean(cicov0,na.rm=T)
estsd = mean(bsds0,na.rm=T)  
results = cbind(ea,n,nsamples,pctsucc,meanb0,medb0,sdb0,adjiqr0,ci0,estsd)
print(results)

print(summary(bet1))
meanb1 = mean(bet1,na.rm=TRUE)
medb1 = median(bet1,na.rm=TRUE)
sdb1 = sd(bet1,na.rm=TRUE)
sumstat1 = summary(bet1)
adjiqr1 = (sumstat1[5]-sumstat1[2])/1.349
ci1 = mean(cicov1,na.rm=T)
estsd = mean(bsds1,na.rm=T)
results = cbind(ea,n,nsamples,pctsucc,meanb1,medb1,sdb1,adjiqr1,ci1,estsd)
print(results)
     




