#LOAD FORTRAN
dyn.load("hwdr.so")

#PARAMETERS FOR MODCHI2 DISTN
mdchmean = 0.8497415
mdchsd = 1.037242

boots = 100
#########################################################################################

#ROUTINE TO GENERATE DATA FOR THE SELECTED SCENARIO
dataset = function (nobs) {

  if (truscen == 1) {
    #NORMAL-NORMAL HOMOSCEDASTIC
    x1 = rnorm(nobs,0,1)
    x2 = rnorm(nobs,0,1)
    z = rnorm(nobs,0,1)   
    oddrat = exp(trubet[1] + trubet[2]*x1 + trubet[3]*x2 + trubet[4]*z)
    pr = oddrat/(1+oddrat)
    y = rbinom(nobs,1,pr)
    condsig = sqrt(2*ea)
    e11 = rnorm(nobs,0,sd=condsig)
    e12 = rnorm(nobs,0,sd=condsig)
    w11 = x1 + e11
    w12 = x1 + e12
    e21 = rnorm(nobs,0,sd=condsig)
    e22 = rnorm(nobs,0,sd=condsig)
    w21 = x2 + e21
    w22 = x2 + e22
   }

  if (truscen == 2) {
    #NORMAL-NORMAL HETEROSCEDASTIC
    x1 = rnorm(nobs,0,1)
    x2 = rnorm(nobs,0,1)
    z = rnorm(nobs,0,1)   
    oddrat = exp(trubet[1] + trubet[2]*x1 + trubet[3]*x2 + trubet[4]*z)
    pr = oddrat/(1+oddrat)
    y = rbinom(nobs,1,pr)
    condsig = sqrt(2*(ea + eb*(abs(x1)+abs(x2)+abs(z))))
    e11 = rnorm(nobs,0,sd=condsig)
    e12 = rnorm(nobs,0,sd=condsig)
    w11 = x1 + e11
    w12 = x1 + e12
    e21 = rnorm(nobs,0,sd=condsig)
    e22 = rnorm(nobs,0,sd=condsig)
    w21 = x2 + e21
    w22 = x2 + e22
   }

   if (truscen == 3) {
     #MODCHI-MODCHI
     sig = sqrt(2*ea)
     urvx1 = 0.9746527*runif(nobs)
     x1 = qchisq(urvx1,1)
     x1 = (x1 - mdchmean)/mdchsd
     urvx2 = 0.9746527*runif(nobs)
     x2 = qchisq(urvx2,1)
     x2 = (x2 - mdchmean)/mdchsd
     z = rnorm(nobs,0,1)
     oddrat = exp(trubet[1] + trubet[2]*x1 + trubet[3]*x2 + trubet[4]*z)
     pr = oddrat/(1+oddrat)
     y = rbinom(nobs,1,pr)    
     urve11 = 0.9746527*runif(nobs)
     e11 = qchisq(urve11,1)
     e11 = (e11 - mdchmean)/mdchsd
     e11 = sig*e11
     w11 = x1 + e11
     urve12 = 0.9746527*runif(nobs)
     e12 = qchisq(urve12,1)
     e12 = (e12 - mdchmean)/mdchsd
     e12 = sig*e12
     w12 = x1 + e12
     e21 = qchisq(urve12,1)
     e21 = (e21 - mdchmean)/mdchsd
     e21 = sig*e21
     w21 = x2 + e21
     urve22 = 0.9746527*runif(nobs)
     e22 = qchisq(urve22,1)
     e22 = (e22 - mdchmean)/mdchsd
     e22 = sig*e22
     w22 = x2 + e22
    } 
     
   dim(y) = c(nobs,1)
   dim(w11) = c(nobs,1)
   dim(w12) = c(nobs,1)
   dim(w21) = c(nobs,1)
   dim(w22) = c(nobs,1)
   dim(z) = c(nobs,1)
   dataset = cbind(y,w11,w12,w21,w22,z)
    
 } 

#########################################################################################

#ROUTINE TO RUN THE HUANG-WANG METHOD

hw = function(n,y,w11,w12,w21,w22,z) {
 
  # naive estimate as starting value
  wavg1 = (w11+w12)/2
  wavg2 = (w21+w22)/2
  naive = glm(y~wavg1+wavg2+z,family=binomial(link='logit'))
  beta0 = naive$coef
 
  # setups for hw method
 
  # npred = number of predictor variables
  # nrep[i,j] = number of replicate person i has on covariate j-1 (intecept = covariate 0)
  # maxrep = max number of replicates
  # derr = error tolerance
  # succ = code for successful estimation
 
  nr0 = rep(1,n)
  nr1 = rep(2,n)
  nr2 = rep(2,n)
  nr3 = rep(1,n)
  # assumes two replicates of w for each person
  nrep = cbind(nr0,nr1,nr2,nr3)
  w = array(rep(1,8*n),dim=c(n,4,2))
  for (i in 1:n) {
    w[i,2,1] = w11[i]
    w[i,2,2] = w12[i]
    w[i,3,1] = w21[i]
    w[i,3,2] = w22[i]
    w[i,4,1] = z[i]
   } 
  npred = 3
  maxrep = 2
  derr = 1e-3
  ndwork = 5*npred + 15
  beta = rep(0,npred+1)
  succ = c(1)
  
  storage.mode(y) = "integer"
  storage.mode(w) = "double"
  storage.mode(nrep) = "integer"
  storage.mode(n) = "integer"
  storage.mode(npred) = "integer"
  storage.mode(maxrep) = "integer"
  storage.mode(beta0) = "double"
  storage.mode(derr) = "double"
  storage.mode(ndwork) = "integer"
  storage.mode(beta) = "double"
  storage.mode(succ) = "integer"
 
  # run hw method
  hwres = .Fortran('hwdr',y,w,nrep,n,npred,maxrep,
    beta0,derr,ndwork,beta=beta,succ=succ)
  estbeta = hwres$beta
  succ = hwres$succ
  results = list(estbeta,succ)
  names(results) = c("estbeta","succ")
  return(results)
  
 }
 
#########################################################################################

#MAIN PROGRAM 
 
n = 500 
nsamples = 1000
ea = 0.4
eb = 0.25
extrfac = 5
seed0 = seq(1,nsamples)+1181
truscen = 2
trubet = c(1,1,1,1)

estbet = matrix(0,nsamples,4)
nsucc = 0
bsds0 = NULL
bsds1 = NULL
bsds2 = NULL
bsds3 = NULL
cicov0 = NULL 
cicov1 = NULL 
cicov2 = NULL 
cicov3 = NULL 
for (isam in 1:nsamples) {
  set.seed(seed0[isam])
  dtaset = dataset(n)
  y = dtaset[,1]
  w11 = dtaset[,2]
  w12 = dtaset[,3]
  w21 = dtaset[,4]
  w22 = dtaset[,5]
  z = dtaset[,6]
  hwres = hw(n,y,w11,w12,w21,w22,z)
  estbeta = hwres$estbeta
  sucval = hwres$succ 
  if (sucval==1) {
    maxabbet = max(abs(estbeta[2:4]))
    if (maxabbet <= extrfac) {    
      nsucc = nsucc + 1 
      estbet[nsucc,] = estbeta
     } 
   }      
  bestbeta =NULL
  #bootstrap 
  for (bb in 1:boots) {
    bindx<-sample(n,replace=T)
    bby = y[bindx]; bw11 = w11[bindx]; bw12 = w12[bindx]; bw21 = w21[bindx]; bw22 = w22[bindx]; bz = z[bindx]
    bhwres = hw(n,bby,bw11,bw12,bw21,bw22,bz)
    bestbeta = rbind(bestbeta,bhwres$estbeta)
  }#end bootstrap
  bsds0[isam] = sd(bestbeta[1,],na.rm=T)
  bsds1[isam] = sd(bestbeta[2,],na.rm=T)
  bsds2[isam] = sd(bestbeta[3,],na.rm=T)
  bsds3[isam] = sd(bestbeta[4,],na.rm=T)
  diff = abs(estbeta[1]-trubet[1]) - 1.96*bsds0[isam]
  cicov0[isam] = (diff <= 0)
  diff = abs(estbeta[2]-trubet[2]) - 1.96*bsds1[isam]
  cicov1[isam] = (diff <= 0)
  diff = abs(estbeta[3]-trubet[3]) - 1.96*bsds2[isam]
  cicov2[isam] = (diff <= 0)
  diff = abs(estbeta[4]-trubet[4]) - 1.96*bsds3[isam]
  cicov3[isam] = (diff <= 0)
 }
    
pctsucc = nsucc/nsamples
beta_mean = apply(estbet,2,mean)
beta_median = apply(estbet,2,median)
beta_sd = apply(estbet,2,sd)
sumstat = apply(estbet,2,summary)
adjiqr = (sumstat[5,]-sumstat[2,])/1.349
print(cbind(ea,n,nsamples,pctsucc))
results = rbind(beta_mean,beta_median,beta_sd,adjiqr)
print(results)

ci = NULL
estsd = NULL
ci[1] = mean(cicov0,na.rm=T)
estsd[1] = mean(bsds0,na.rm=T)
ci[2] = mean(cicov1,na.rm=T)
estsd[2] = mean(bsds1,na.rm=T)
ci[3] = mean(cicov2,na.rm=T)
estsd[3] = mean(bsds2,na.rm=T)
ci[4] = mean(cicov3,na.rm=T)
estsd[4] = mean(bsds3,na.rm=T)

print(estsd)
print(ci)


 
     

