c     NON-Parametrically corrected estimating function for
c     LOGISTIC regression
c     NOTE: for nonparametric-correction, alpha cannot be
c     consistently estimated

c     subroutines DGECO and DGESL from CMLIB are used
c     (see document on DGEFS)

c     !!!! beta(-1:npred) !!!!
      subroutine npc(y,w,ptmass,nrep,size,npred,maxrep,
     &     beta,derr,dwork,ndwork,ipvt,succ)
      integer size,y(size),ptmass(size),nrep(size,0:npred),
     &     npred,maxrep,ndwork,ipvt(0:npred,2),succ
      double precision w(size,0:npred,maxrep),
     &     beta(-1:npred),dwork(0:npred,ndwork)
c     y(size)                - outcome 0/1 variable
c                              Pr(y=1 | x)=Psi(beta^t * x) (INPUT)
c     w(size,0:npred,maxrep) - surrogate covariates=x+mean-0
c                                                     normal (INPUT)
c     ptmass(size)           - weight mass assigned to each point:
c                              for any given sample, it is straight 1's
c                              ptmass is introduced in consideration of
c                              bootstrap (INPUT)
c     nrep(size,0:npred)     - number of repeated measures; number=1
c                              across all participants implies that
c                              the covariate is accurately measured (INPUT)
c     size                   - sample size (INPUT)
c     npred                  - number of covariates (INPUT)
c     maxrep                 - maximum number of replications (INPUT)
c     beta(-1:npred)         - estimated covariate coefficient (OUTPUT)
c     derr                   - error tolerance (INPUT)
c     dwork(0:npred,ndwork)  - working array
c     ndwork                 - =5*npred+15
c     ipvt(0:npred,2)        - working array
c     succ                   - (OUTPUT)

      external npc_omin,npc_obj
      double precision npc_omin,npc_obj
      integer p,q,n,m,i
      double precision dtmp

      dwork = 0.0d0
      do i=1,2
c     estimate the variance of the estimating functions
         if(i .eq. 1) then
            do p=0,npred
               do n=1,2
                  do q=0,npred
                     do m=1,2
                        dwork(p,n+q*2+(m-1)*(npred+1)*2)=0.0d0
                     enddo
                  enddo
                  dwork(p,n+p*2+(n-1)*(npred+1)*2)=1.0d0
               enddo
            enddo
         else
            call npc_vest(y,w,ptmass,nrep,size,npred,maxrep,
     &           beta,dwork(0,4*npred+5),dwork)
         endif

         CALL DGECO(dwork,(npred+1)*2,(npred+1)*2,ipvt,
     &        dtmp,dwork(0,4*npred+5))

c     point estimator
         dtmp=dtmp*dble(size)
         if(dtmp .gt. 1.0d-6)
     &        dtmp=npc_omin(y,w,ptmass,nrep,size,npred,
     &        maxrep,beta,dwork,derr,ipvt,dwork(0,4*npred+5),
     &        dwork(0,4*npred+9),dwork(0,5*npred+14),succ)

      enddo

      beta(0)=(beta(0)+beta(-1))/2.0d0

      end

c     ********************************************************
c     variance estimation of the two estimating functions
c     amenable to correction:

      subroutine npc_vest(y,w,ptmass,nrep,size,npred,maxrep,
     &     b,f,fv)
      integer size,y(size),ptmass(size),nrep(size,0:npred),
     &     npred,maxrep
      double precision w(size,0:npred,maxrep),
     &     b(-1:npred),f(0:npred,2,2),fv(0:npred,2,0:npred,2)
c     b(-1:npred)            - dummy variables in the est. fun. (INPUT)
c     f(0:npred,2,2)         - working
c     fv(0:npred,2,0:npred,2)- inverse variance estimate of est. fun. (OUTPUT)

      integer i,j,p,q,n,m
      double precision sum,dtmp1,dtmp2,dtmp3,dtmp4

      do p=0,npred
         do n=1,2
            f(p,n,1)=0.0d0
            do q=0,npred
               do m=1,2
                  fv(p,n,q,m)=0.0d0
               enddo
            enddo
         enddo
      enddo
      do i=1,size
         if(ptmass(i) .gt. 0) then
            do q=0,npred
               f(q,1,2)=dexp(-b(-1))
               f(q,2,2)=dexp(b(0))
            enddo
            do p=1,npred
               sum=0.0d0
               do j=1,nrep(i,p)
                  sum=sum+w(i,p,j)
               enddo
               if(nrep(i,p) .eq. 1) then
                  dtmp1=dexp(-b(p)*w(i,p,1))
                  dtmp2=dtmp1*w(i,p,1)
                  dtmp3=dexp(b(p)*w(i,p,1))
                  dtmp4=dtmp3*w(i,p,1)
               else
                  dtmp1=0.0d0
                  dtmp2=0.0d0
                  dtmp3=0.0d0
                  dtmp4=0.0d0
                  do j=1,nrep(i,p)
                     dtmp1=dtmp1+dexp(-b(p)*w(i,p,j))
                     dtmp2=dtmp2+dexp(-b(p)*w(i,p,j))
     &                    *(sum-w(i,p,j))
                     dtmp3=dtmp3+dexp(b(p)*w(i,p,j))
                     dtmp4=dtmp4+dexp(b(p)*w(i,p,j))
     &                    *(sum-w(i,p,j))
                  enddo
                  dtmp1=dtmp1/dble(nrep(i,p))
                  dtmp2=dtmp2/dble(nrep(i,p)*(nrep(i,p)-1))
                  dtmp3=dtmp3/dble(nrep(i,p))
                  dtmp4=dtmp4/dble(nrep(i,p)*(nrep(i,p)-1))
               endif
               do q=0,npred
                  if(q .ne. p) then
                     f(q,1,2)=f(q,1,2)*dtmp1
                     f(q,2,2)=f(q,2,2)*dtmp3
                  else
                     f(q,1,2)=f(q,1,2)*dtmp2
                     f(q,2,2)=f(q,2,2)*dtmp4
                  endif
               enddo
            enddo
            do p=0,npred
               sum=0.0d0
               do j=1,nrep(i,p)
                  sum=sum+w(i,p,j)
               enddo
               f(p,1,2)=dble(y(i)-1)*sum/dble(nrep(i,p))
     &              +dble(y(i))*f(p,1,2)
               f(p,2,2)=dble(y(i))*sum/dble(nrep(i,p))
     &              +dble(y(i)-1)*f(p,2,2)
            enddo

            do p=0,npred
               do n=1,2
                  f(p,n,1)=f(p,n,1)+f(p,n,2)
     &                 *dble(ptmass(i))
                  do q=0,npred
                     do m=1,2
                        fv(p,n,q,m)=fv(p,n,q,m)
     &                       +f(p,n,2)*f(q,m,2)
     &                       *dble(ptmass(i))
                     enddo
                  enddo
               enddo
            enddo
         endif
      enddo

      do p=0,npred
         do n=1,2
            do q=0,npred
               do m=1,2
                  fv(p,n,q,m)=fv(p,n,q,m)
     &                 /dble(size)**2.0d0
               enddo
            enddo
         enddo
      enddo

      end

c     ********************************************************
c     Parametric-correction with known variance
c     minimized objective function
      double precision function
     &     npc_omin(y,w,ptmass,nrep,size,npred,maxrep,
     &           beta,fv,derr,ipvt,f,init,out,succ)
      integer size,y(size),ptmass(size),nrep(size,0:npred),
     &     npred,maxrep,ipvt(0:npred,2),succ
      double precision w(size,0:npred,maxrep),
     &     beta(-1:npred),fv(0:npred,2,0:npred,2),derr,
     &     f(0:npred,4),
     &     init(-2:npred,-1:npred),out(-2:npred)

      external npc_obj
      double precision npc_obj
      integer iter,i,j

c     initialization
      do i=-2,npred
         do j=-1,npred
            init(i,j)=0.0d0
         enddo
      enddo
      do i=-1,npred
         init(i,i)=init(i,i)+5.0d-1
      enddo

c     +++++++
c      init(npred,0)=5.5d0
c      init(npred,1)=-1.0d0
c     +++++++

      do i=-2,npred
         do j=-1,npred
            beta(j)=init(i,j)
         enddo
         out(i)=npc_obj(beta,
     &     y,w,ptmass,nrep,size,npred,maxrep,f,fv,ipvt)
      enddo
      call nAMOEBA(init,out,npred+3,npred+2,npred+2,derr,
     &     npc_obj,iter,
     &     y,w,ptmass,nrep,size,npred,maxrep,f,fv,ipvt,
     &     succ)
      i=-2
      do j=-1,npred
         if(out(j) .lt. out(i)) i=j
      enddo
      npc_omin=out(i)
      do j=-1,npred
         beta(j)=init(i,j)
      enddo
      end

c     ********************************************************
c     the objective function
      double precision function npc_obj(b,
     &     y,w,ptmass,nrep,size,npred,maxrep,f,fv,ipvt)
      double precision b(-1:npred)
      integer size,y(size),ptmass(size),nrep(size,0:npred),
     &     npred,maxrep,ipvt(0:npred,2)
      double precision w(size,0:npred,maxrep),
     &     f(0:npred,2,2),fv(0:npred,2,0:npred,2)
c     b(-1:npred)            - dummy variables in the est. fun. (INPUT)
c     f(0:npred,2,2)         - working
c     fv(0:npred,2,0:npred,2)- inverse variance estimate of est. fun. (INPUT)
c     ipvt(0:npred,2)        - working

      integer i,j,p,q,arm
      double precision sum,dtmp1,dtmp2,dtmp3,dtmp4

      do arm=1,2
         do p=0,npred
            f(p,arm,1)=0.0d0
         enddo
      enddo
      do i=1,size
         if(ptmass(i) .gt. 0) then
            do q=0,npred
               f(q,1,2)=dexp(-b(-1))
               f(q,2,2)=dexp(b(0))
            enddo
            do p=1,npred
               sum=0.0d0
               do j=1,nrep(i,p)
                  sum=sum+w(i,p,j)
               enddo
               if(nrep(i,p) .eq. 1) then
                  dtmp1=dexp(-b(p)*w(i,p,1))
                  dtmp2=dtmp1*w(i,p,1)
                  dtmp3=dexp(b(p)*w(i,p,1))
                  dtmp4=dtmp3*w(i,p,1)
               else
                  dtmp1=0.0d0
                  dtmp2=0.0d0
                  dtmp3=0.0d0
                  dtmp4=0.0d0
                  do j=1,nrep(i,p)
                     dtmp1=dtmp1+dexp(-b(p)*w(i,p,j))
                     dtmp2=dtmp2+dexp(-b(p)*w(i,p,j))
     &                    *(sum-w(i,p,j))
                     dtmp3=dtmp3+dexp(b(p)*w(i,p,j))
                     dtmp4=dtmp4+dexp(b(p)*w(i,p,j))
     &                    *(sum-w(i,p,j))
                  enddo
                  dtmp1=dtmp1/dble(nrep(i,p))
                  dtmp2=dtmp2/dble(nrep(i,p)*(nrep(i,p)-1))
                  dtmp3=dtmp3/dble(nrep(i,p))
                  dtmp4=dtmp4/dble(nrep(i,p)*(nrep(i,p)-1))
               endif
               do q=0,npred
                  if(q .ne. p) then
                     f(q,1,2)=f(q,1,2)*dtmp1
                     f(q,2,2)=f(q,2,2)*dtmp3
                  else
                     f(q,1,2)=f(q,1,2)*dtmp2
                     f(q,2,2)=f(q,2,2)*dtmp4
                  endif
               enddo
            enddo
            do p=0,npred
               sum=0.0d0
               do j=1,nrep(i,p)
                  sum=sum+w(i,p,j)
               enddo
               f(p,1,2)=dble(y(i)-1)*sum/dble(nrep(i,p))
     &              +dble(y(i))*f(p,1,2)
               f(p,2,2)=dble(y(i))*sum/dble(nrep(i,p))
     &              +dble(y(i)-1)*f(p,2,2)
            enddo

            do p=0,npred
               do arm=1,2
                  f(p,arm,1)=f(p,arm,1)
     &                 +f(p,arm,2)*dble(ptmass(i))
               enddo
            enddo
         endif
      enddo

      do arm=1,2
         do p=0,npred
            f(p,arm,1)=f(p,arm,1)/dble(size)
            f(p,arm,2)=f(p,arm,1)
         enddo
      enddo

      CALL DGESL(fv,(npred+1)*2,(npred+1)*2,ipvt,
     &     f(0,1,2),0)

c     objective function
      npc_obj=0.0d0
      do arm=1,2
         do p=0,npred
            npc_obj=npc_obj+f(p,arm,1)*f(p,arm,2)
         enddo
      enddo

      end

      SUBROUTINE nAMOEBA(P,Y,MP,NP,NDIM,FTOL,FUNK,ITER,
     &     yy,w,ptmass,nrep,size,npred,maxrep,f,fv,ipvt,
     &     succ)

      implicit double precision(a-h,o-z)
      integer size,yy(size),ptmass(size),nrep(size,0:npred),
     &     npred,maxrep,ipvt(0:npred,2),succ
      double precision w(size,0:npred,maxrep),
     &     f(0:npred,2,2),fv(0:npred,2,0:npred,2)

      PARAMETER (NMAX=20,ALPHA=1.0d0,BETA=0.5d0,GAMMA=2.0d0,
     &     ITMAX=500)
      DIMENSION P(MP,NP),Y(MP),PR(NMAX),PRR(NMAX),PBAR(NMAX)

      succ=1

      MPTS=NDIM+1
      ITER=0
1     ILO=1
      IF(Y(1).GT.Y(2))THEN
        IHI=1
        INHI=2
      ELSE
        IHI=2
        INHI=1
      ENDIF
      DO 11 I=1,MPTS
        IF(Y(I).LT.Y(ILO)) ILO=I
        IF(Y(I).GT.Y(IHI))THEN
          INHI=IHI
          IHI=I
        ELSE IF(Y(I).GT.Y(INHI))THEN
          IF(I.NE.IHI) INHI=I
        ENDIF
11    CONTINUE
      RTOL=2.0d0*dABS(Y(IHI)-Y(ILO))/(dABS(Y(IHI))+dABS(Y(ILO)))
      IF(RTOL.LT.FTOL)RETURN
      IF(ITER.EQ.ITMAX) then
         succ=0
         RETURN
      endif
      ITER=ITER+1
      DO 12 J=1,NDIM
        PBAR(J)=0.0d0
12    CONTINUE
      DO 14 I=1,MPTS
        IF(I.NE.IHI)THEN
          DO 13 J=1,NDIM
            PBAR(J)=PBAR(J)+P(I,J)
13        CONTINUE
        ENDIF
14    CONTINUE
      DO 15 J=1,NDIM
        PBAR(J)=PBAR(J)/NDIM
        PR(J)=(1.+ALPHA)*PBAR(J)-ALPHA*P(IHI,J)
15    CONTINUE
      YPR=FUNK(PR,
     &     yy,w,ptmass,nrep,size,npred,maxrep,
     &     f,fv,ipvt)
      IF(YPR.LE.Y(ILO))THEN
        DO 16 J=1,NDIM
          PRR(J)=GAMMA*PR(J)+(1.-GAMMA)*PBAR(J)
16      CONTINUE
        YPRR=FUNK(PRR,
     &       yy,w,ptmass,nrep,size,npred,maxrep,
     &       f,fv,ipvt)
        IF(YPRR.LT.Y(ILO))THEN
          DO 17 J=1,NDIM
            P(IHI,J)=PRR(J)
17        CONTINUE
          Y(IHI)=YPRR
        ELSE
          DO 18 J=1,NDIM
            P(IHI,J)=PR(J)
18        CONTINUE
          Y(IHI)=YPR
        ENDIF
      ELSE IF(YPR.GE.Y(INHI))THEN
        IF(YPR.LT.Y(IHI))THEN
          DO 19 J=1,NDIM
            P(IHI,J)=PR(J)
19        CONTINUE
          Y(IHI)=YPR
        ENDIF
        DO 21 J=1,NDIM
          PRR(J)=BETA*P(IHI,J)+(1.0d0-BETA)*PBAR(J)
21      CONTINUE
        YPRR=FUNK(PRR,
     &       yy,w,ptmass,nrep,size,npred,maxrep,
     &             f,fv,ipvt)
        IF(YPRR.LT.Y(IHI))THEN
          DO 22 J=1,NDIM
            P(IHI,J)=PRR(J)
22        CONTINUE
          Y(IHI)=YPRR
        ELSE
          DO 24 I=1,MPTS
            IF(I.NE.ILO)THEN
              DO 23 J=1,NDIM
                PR(J)=0.5d0*(P(I,J)+P(ILO,J))
                P(I,J)=PR(J)
23            CONTINUE
              Y(I)=FUNK(PR,
     &             yy,w,ptmass,nrep,size,npred,maxrep,
     &             f,fv,ipvt)
            ENDIF
24        CONTINUE
        ENDIF
      ELSE
        DO 25 J=1,NDIM
          P(IHI,J)=PR(J)
25      CONTINUE
        Y(IHI)=YPR
      ENDIF
      GO TO 1
      END
