# COMPUTER CODE

# A Regularization Corrected Score Method for
# Nonlinear Regression Models with Covariate Error
# David M. Zucker, Malka Gorfine, Yi Li, Mahlet G. Tadesse, and Donna Spiegelman
# Published in Biometrics

# CODE TO RUN SIMULATIONS OF N&S METHOD (MULTIVARIATE)

library(nleqslv)
#library("nleqslv",lib.loc="/udd/stzuc/R/sparc-sun-solaris2.8-library/2.8/")
options(width=200) #to get long line

#clean up working environment from past runs
rm(list = ls())
graphics.off()

n = 500
trub0 = 1
trub1 = 1
trub2 = 1
trub3 = 1
trubet = c(trub0,trub1, trub2, trub3)
k = 100
samples = 1000
extrfac = 5
xtol = 1e-6
ftol = 2e-3
btol = 1e-3
maxit = 200
seed0 = seq(1,samples)+11817
truscen = 3
ea = 1
eb = NULL
if (truscen == 2) {
  ea = 0.4
  eb = 0.25
 }
var1 = 1
var2 = 1

#PARAMETERS FOR MODCHI2 DISTN
mdchmean = 0.8497415
mdchsd = 1.037242

inputs = cbind(n,trub0,trub1,trub2,trub3,truscen,ea,eb,k,samples)
print(inputs)
inputs2 = cbind(xtol,ftol,btol,maxit)
print(inputs2)

wuns_k = rep(1,k)
wuns_n = rep(1,n)
dim(wuns_n) = c(n,1)
sig1 = sqrt(var1)
sig2 = sqrt(var2)

#########################################################################################

#ROUTINE TO GENERATE DATA FOR THE SELECTED SCENARIO
dataset = function (nobs) {

  if (truscen == 1) {
    #NORMAL-NORMAL HOMOSCEDASTIC
    x1 = rnorm(nobs,0,1)
    x2 = rnorm(nobs,0,1)
    z = rnorm(nobs,0,1)   
    oddrat = exp(trubet[1] + trubet[2]*x1 + trubet[3]*x2 + trubet[4]*z)
    pr = oddrat/(1+oddrat)
    y = rbinom(nobs,1,pr)
    condsig = sqrt(ea)
    e1 = rnorm(nobs,0,sd=condsig)
    e2 = rnorm(nobs,0,sd=condsig)
    w1 = x1 + e1
    w2 = x2 + e2
   }

  if (truscen == 2) {
    #NORMAL-NORMAL HETEROSCEDASTIC
    x1 = rnorm(nobs,0,1)
    x2 = rnorm(nobs,0,1)
    z = rnorm(nobs,0,1)   
    oddrat = exp(trubet[1] + trubet[2]*x1 + trubet[3]*x2 + trubet[4]*z)
    pr = oddrat/(1+oddrat)
    y = rbinom(nobs,1,pr)
    condsig = sqrt(ea + eb*(abs(x1)+abs(x2)+abs(z)))
    e1 = rnorm(nobs,0,sd=condsig)
    e2 = rnorm(nobs,0,sd=condsig)
    w1 = x1 + e1
    w2 = x2 + e2
   }

   if (truscen == 3) {
     #MODCHI-MODCHI
     sig = sqrt(ea)
     urvx1 = 0.9746527*runif(nobs)
     x1 = qchisq(urvx1,1)
     x1 = (x1 - mdchmean)/mdchsd
     urvx2 = 0.9746527*runif(nobs)
     x2 = qchisq(urvx2,1)
     x2 = (x2 - mdchmean)/mdchsd
     z = rnorm(nobs,0,1)
     oddrat = exp(trubet[1] + trubet[2]*x1 + trubet[3]*x2 + trubet[4]*z)
     pr = oddrat/(1+oddrat)
     y = rbinom(nobs,1,pr)    
     urve1 = 0.9746527*runif(nobs)
     e1 = qchisq(urve1,1)
     e1 = (e1 - mdchmean)/mdchsd
     e1 = sig*e1
     w1 = x1 + e1
     urve2 = 0.9746527*runif(nobs)
     e2 = qchisq(urve2,1)
     e2 = (e2 - mdchmean)/mdchsd
     e2 = sig*e2
     w2 = x2 + e2
    } 
     
   dim(y) = c(nobs,1)
   dim(w1) = c(nobs,1)
   dim(w2) = c(nobs,1)
   dim(z) = c(nobs,1)
   dataset = cbind(y,w1,w2,z)
    
 } 

#########################################################################################

# CORRECTED SCORE FUNCTION
cscmp = function(beta,y,w1,w2,z,szeta1,szeta2,flg) {

  b1 = beta[2]
  b2 = beta[3]
  vec_ans = rep(0,4)
  mat_ans = matrix(0,4,4)
  cmat = cbind(wuns_n,w1,w2,z)
  ewv = exp(cmat%*%beta)
  w1mat = w1%*%wuns_k
  w2mat = w2%*%wuns_k
  ew = ewv%*%wuns_k   
  
  a = ew * cos(b1*szeta1+b2*szeta2)
  b = ew * sin(b1*szeta1+b2*szeta2)
  denom = (1+a)^2 + b^2
  f = (a*(1+a)+b^2)/denom
  g = b/denom
  hh1 = szeta1*g
  hh2 = szeta2*g
  
  fmean = rowSums(f)/k
  hh1mean = rowSums(hh1)/k
  hh2mean = rowSums(hh2)/k

  u1sv = y-fmean
  u2sv = w1*(y-fmean) + hh1mean
  u3sv = w2*(y-fmean) + hh2mean  
  u4sv = z*(y-fmean)
  
  u1s = mean(u1sv)
  u2s = mean(u2sv)
  u3s = mean(u3sv)  
  u4s = mean(u4sv)
  vec_ans = c(u1s,u2s,u3s,u4s)
  
  if (flg == 2) { 
    umat = cbind(u1sv,u2sv,u3sv,u4sv)
    mat_ans = t(umat) %*% umat / n
   }    
   
  results = list(vec_ans,mat_ans)
  names(results) = c('vec_ans','mat_ans')
  return(results)
  
 }

corscor = function(beta,y,w1,w2,z,szeta1,szeta2) {
  csres = cscmp(beta,y,w1,w2,z,szeta1,szeta2,1)
  corscor = csres$vec_ans
 } 

#########################################################################################

# JACOBIAN OF CORRECTED SCORE FUNCTION
csjac = function(beta,y,w1,w2,z,szeta1,szeta2) {

  jacmat = matrix(0,4,4)

  b1 = beta[2]
  b2 = beta[3]
  cmat = cbind(wuns_n,w1,w2,z)
  ewv = exp(cmat%*%beta)
  w1mat = w1%*%wuns_k
  w2mat = w2%*%wuns_k
  zmat = z%*%wuns_k
  ew = ewv%*%wuns_k   
  
  a = ew * cos(b1*szeta1+b2*szeta2)
  b = ew * sin(b1*szeta1+b2*szeta2)
  denom = (1+a)^2 + b^2
  f = (a*(1+a)+b^2)/denom
  g = b/denom
  hh1 = szeta1*g
  hh2 = szeta2*g
  
  fmean = rowSums(f)/k
  hh1mean = rowSums(hh1)/k
  hh2mean = rowSums(hh2)/k
  
  dadb0 = a
  dadb1 = w1mat*a - szeta1*b
  dadb2 = w2mat*a - szeta2*b
  dadb3 = zmat*a
   
  dbdb0 = b
  dbdb1 = w1mat*b + szeta1*a
  dbdb2 = w2mat*b + szeta2*a
  dbdb3 = zmat*b  
  
  dfdb0 = ((1+2*a)*dadb0+2*b*dbdb0)/denom - f*(2*(1+a)*dadb0+2*b*dbdb0)/denom
  dfdb1 = ((1+2*a)*dadb1+2*b*dbdb1)/denom - f*(2*(1+a)*dadb1+2*b*dbdb1)/denom
  dfdb2 = ((1+2*a)*dadb2+2*b*dbdb2)/denom - f*(2*(1+a)*dadb2+2*b*dbdb2)/denom
  dfdb3 = ((1+2*a)*dadb3+2*b*dbdb3)/denom - f*(2*(1+a)*dadb3+2*b*dbdb3)/denom
  
  dgdb1 = dbdb1/denom - g*(2*(1+a)*dadb1+2*b*dbdb1)/denom
  dgdb2 = dbdb2/denom - g*(2*(1+a)*dadb2+2*b*dbdb2)/denom
  dgdb3 = dbdb3/denom - g*(2*(1+a)*dadb3+2*b*dbdb3)/denom
  
  dh1db1 = w1mat*dfdb1 - szeta1*dgdb1
  dh1db2 = w1mat*dfdb2 - szeta1*dgdb2
  dh1db3 = w1mat*dfdb3 - szeta1*dgdb3
  dh2db1 = w2mat*dfdb1 - szeta2*dgdb1
  dh2db2 = w2mat*dfdb2 - szeta2*dgdb2
  dh2db3 = w2mat*dfdb3 - szeta2*dgdb3

  jacmat[1,1] = -mean(rowSums(dfdb0)/k)
  jacmat[1,2] = -mean(rowSums(dfdb1)/k)
  jacmat[1,3] = -mean(rowSums(dfdb2)/k)
  jacmat[1,4] = -mean(rowSums(dfdb3)/k)
  jacmat[2,2] = -mean(rowSums(dh1db1)/k)
  jacmat[2,3] = -mean(rowSums(dh1db2)/k)
  jacmat[2,4] = -mean(rowSums(dh1db3)/k) 
  jacmat[3,3] = -mean(rowSums(dh2db2)/k)
  jacmat[3,4] = -mean(rowSums(dh2db3)/k)
  jacmat[4,4] = -mean(rowSums(zmat*dfdb3)/k) 
  jacmat[2,1] = jacmat[1,2]
  jacmat[3,1] = jacmat[1,3]
  jacmat[4,1] = jacmat[1,4]
  jacmat[3,2] = jacmat[2,3]
  jacmat[4,2] = jacmat[2,4]
  jacmat[4,3] = jacmat[3,4]  
  return(jacmat)

 } 
#END JACOBIAN OF CORRECTED SCORE FUNCTION 

#########################################################################################
 
#MAIN PROGRAM 

#SET UP RESULTS MATRICES
estimates = array(rep(NA,8*samples),dim=c(samples,4,2))
sds = array(rep(NA,8*samples),dim=c(samples,4,2))
cicov = array(rep(NA,8*samples),dim=c(samples,4,2))
estmat = matrix(NA,4,2)
emmat = matrix(NA,4,2)
sdmat = matrix(NA,4,2)
sdmmat = matrix(NA,4,2)
empsdmat = matrix(NA,4,2)
iqrmat = matrix(NA,4,2)
cimat = matrix(NA,4,2)
gudnum = c(0,0)

#SIMULATION LOOP
for (i in 1:samples) {

#GENERATE DATA
set.seed(seed0[i])
dtaset = dataset(n)
y = dtaset[,1]
w1 = dtaset[,2]
w2 = dtaset[,3]
w = dtaset[,2:3]
z = dtaset[,4]
 
# GENERATE ZETA'S
zeta1 = rnorm(n,0,1)
zeta2 = rnorm(n,0,1)
for (kk in 2:k) {
  zz1 = rnorm(n,0,1)
  zeta1 = cbind(zeta1,zz1)
  zz2 = rnorm(n,0,1)
  zeta2 = cbind(zeta2,zz2)
 }
szeta1 = sig1*zeta1
szeta2 = sig2*zeta2 

# NAIVE ESTIMATE
naive = glm(y~w1+w2+z,family=binomial)
betnai = naive$coef
covnai = summary(naive)$cov.unscaled
abmax = max(abs(betnai[2:4]))
if (abmax <= extrfac) {
  gudnum[1] = gudnum[1] + 1
  for (rr in 1:4) {
    estimates[i,rr,1] = betnai[rr]
    sds[i,rr,1] = sqrt(covnai[rr,rr])
    diff = abs(estimates[i,rr,1]-trubet[rr]) - 1.96*sds[i,rr,1]
    cicov[i,rr,1] = (diff <= 0)
   }
 } 

# COMPUTE CORRECTED SCORE ESTIMATOR
dim(w1) = c(n,1)
dim(w2) = c(n,1)
dim(z) = c(n,1)
beta = betnai      
cntl = list(xtol,ftol,btol,maxit)
names(cntl) = c('xtol','ftol','btol','maxit')
funest = nleqslv(beta,corscor,jac=csjac,y=y,w1=w1,w2=w2,z=z,szeta1=szeta1,szeta2=szeta2,control=cntl)
betvec = funest$x
fvec = funest$fvec
termcd = funest$termcd
fvamax = max(abs(fvec))
good = (fvamax <= ftol)
abmax = max(abs(betvec[2:4]))

# WEED OUT BAD/EXTREME ESTIMATES
if (good & (abmax <= extrfac)) {

  gudnum[2]= gudnum[2] + 1

  #compute theoretical variances
  jacmat = csjac(betvec,y,w1,w2,z,szeta1,szeta2)
  didl = 1e-4
  kap = kappa(jacmat, exact=TRUE)
  jacinv = solve(jacmat)
  csres = cscmp(betvec,y,w1,w2,z,szeta1,szeta2,2)
  filling = csres$mat_ans
  theocov = jacinv %*% filling %*% jacinv / n

  # derivative check   
  #pert = c(0,0,0,didl)
  #bbbl = betvec-pert
  #csrl = cscmp(bbbl,y,w1,w2,z,szeta1,szeta2,1)
  #c1 = csrl$vec_ans
  #bbbu = betvec+pert
  #csru = cscmp(bbbu,y,w1,w2,z,szeta1,szeta2,1)
  #c2 = csru$vec_ans
  #deri = (c2-c1)/(2*didl)
  
  #store results
  for (rr in 1:4) {
    estimates[i,rr,2] = betvec[rr]
    sds[i,rr,2] = sqrt(theocov[rr,rr])
    diff = abs(estimates[i,rr,2]-trubet[rr]) - 1.96*sds[i,rr,2]
    cicov[i,rr,2] = (diff <= 0)
   }
 }
} 

#SUMMARY STATISTICS
pctgud = gudnum/samples
for (imeth in 1:2) {
for (rr in 1:4) {
  estvec = estimates[,rr,imeth]
  sdvec = sds[,rr,imeth]
  civec = cicov[,rr,imeth]
  #print("Statistics on Estimates")
  #print(stem(estvec))
  #print(summary(estvec,na.rm=TRUE))
  #print("Statistics on Theoretical SD's")
  #print(stem(sdvec))
  #print(summary(sdvec,na.rm=TRUE))  
  estmat[rr,imeth] = mean(estvec,na.rm=TRUE)
  emmat[rr,imeth] = median(estvec,na.rm=TRUE)
  sdmat[rr,imeth] = mean(sdvec,na.rm=TRUE)
  sdmmat[rr,imeth] = median(sdvec,na.rm=TRUE)
  empsdmat[rr,imeth] = sd(estvec,na.rm=TRUE)
  sumstat = summary(estvec,na.rm=TRUE)
  iqrmat[rr,imeth] = (sumstat[5]-sumstat[2])/1.349
  cimat[rr,imeth] = mean(civec,na.rm=TRUE)
 }
 }

gud = pctgud
ee = data.frame(estmat)
eem = data.frame(emmat)
ss1 = data.frame(sdmat)
ss1a = data.frame(sdmmat)
ss2 = data.frame(empsdmat)
iqr = data.frame(iqrmat)
ci = data.frame(cimat)
colname = c("naive","mc-ce")
names(gud) = colname
names(ee) = colname
names(eem) = colname
names(ss1) = colname
names(ss1a) = colname
names(ss2) = colname
names(iqr) = colname
names(ci) = colname
print("Percent of Samples with Successful Estimation")
print(gud)
print("Mean of Estimates")
print(ee)
print("Median of Estimates")
print(eem)
print("Mean Estimated SD")
print(ss1)
print("Median Estimated SD")
print(ss1a)
print("Empirical SD of Estimates")
print(ss2)
print("IQR/1.349")
print(iqr)   
print("Confidence Interval Coverage")
print(ci)
