library(simex)
library(nleqslv)
#library("nleqslv",lib.loc="/udd/stzuc/R/sparc-sun-solaris2.8-library/2.8/")
options(width=200) #to get long line

#clean up working environment from past runs
rm(list = ls())
graphics.off()

n = 500
trub0 = 1
trub1 = 1
trub2 = 1
trub3 = 1
k = 100
var1 = 1
var2 = 1
ea = 1
eb = NULL
samples = 1000
#truscen = 1 #truscen 1 goes with ea = 1 eb = NULL
#truscen = 2 #truscen 2 goes with ea = 0.4 eb = 0.2
truscen = 3 #truscen 3 goes with ea = 1 eb = NULL 
seed0 = seq(1,samples)+11817 
inputs = cbind(n,trub0,trub1,trub2,trub3,var1,var2,k,samples)
print(inputs)

#PARAMETERS FOR MODCHI2 DISTN
mdchmean = 0.8497415
mdchsd = 1.037242

wuns_k = rep(1,k)
wuns_n = rep(1,n)
dim(wuns_n) = c(n,1)
sig1 = sqrt(var1)
sig2 = sqrt(var2)

# CORRECTED SCORE FUNCTION
corscor = function(beta,y,w1,w2,z,szeta1,szeta2) {

  b1 = beta[2]
  b2 = beta[3]
  cmat = cbind(wuns_n,w1,w2,z)
  ewv = exp(cmat%*%beta)
  w1mat = w1%*%wuns_k
  w2mat = w2%*%wuns_k
  ew = ewv%*%wuns_k   
  
  a = ew * cos(b1*szeta1+b2*szeta2)
  b = ew * sin(b1*szeta1+b2*szeta2)
  denom = (1+a)^2 + b^2
  f = (a*(1+a)+b^2)/denom
  g = b/denom
  hh1 = szeta1*g
  hh2 = szeta2*g
  
  fmean = rowSums(f)/k
  hh1mean = rowSums(hh1)/k
  hh2mean = rowSums(hh2)/k
  
  u1s = mean(y-fmean)
  u2s = mean(w1*(y-fmean)+hh1mean)
  u3s = mean(w2*(y-fmean)+hh2mean)  
  u4s = mean(z*(y-fmean))
  
  corscor = c(u1s,u2s,u3s,u4s)
  
 }
#END CORRECTED SCORE FUNCTION
 
#MAIN PROGRAM 

estimates = array(rep(0,20*samples),dim=c(samples,4,5))
sds = array(rep(0,20*samples),dim=c(samples,4,5))
cicov = array(rep(0,20*samples),dim=c(samples,4,5))
medmat = matrix(0,4,5)
estmat = matrix(0,4,5)
emmat = matrix(0,4,5)
sdmat = matrix(0,4,5)
sdmmat = matrix(0,4,5)
empsdmat = matrix(0,4,5)
iqrmat = matrix(0,4,5)
cimat = matrix(0,4,5)
gudnum = c(0,0,0,0,0)
nacount =matrix(0,4,5)



for (i in 1:samples) {

#GENERATE DATA
set.seed(seed0[i])

  if (truscen == 1) {
    # Case1: normal-normal homo
    x1 = rnorm(n,0,1)
    x2 = rnorm(n,0,1)
    z = rnorm(n,0,1)
    oddrat = exp(trub0 + trub1*x1 + trub2*x2 + trub1*z)
    pr = oddrat/(1+oddrat)
    y = rbinom(n,1,pr)
    e1 = rnorm(n,0,sd=sig1)
    e2 = rnorm(n,0,sd=sig2)
    w1 = x1 + e1
    w2 = x2 + e2
    w = cbind(w1,w2)
  }

  if (truscen == 2) {
    nobs <- n
    trubet <- c(trub0,trub1,trub2,trub3)
    # Case1: normal-normal hetero
    x1 = rnorm(nobs,0,1)
    x2 = rnorm(nobs,0,1)
    z = rnorm(nobs,0,1)
    oddrat = exp(trubet[1] + trubet[2]*x1 + trubet[3]*x2 + trubet[4]*z)
    pr = oddrat/(1+oddrat)
    y = rbinom(nobs,1,pr)
    condsig = sqrt((ea + eb*(abs(x1)+abs(x2)+abs(z))))
    e1 = rnorm(nobs,0,sd=condsig)
    e2 = rnorm(nobs,0,sd=condsig)
    w1 = x1 + e1
    w2 = x2 + e2
  }

  if (truscen == 3) {
     nobs <- n
     trubet <- c(trub0,trub1,trub2,trub3)
     #MODCHI-MODCHI
     sig = sqrt(ea)
     urvx1 = 0.9746527*runif(nobs)
     x1 = qchisq(urvx1,1)
     x1 = (x1 - mdchmean)/mdchsd
     urvx2 = 0.9746527*runif(nobs)
     x2 = qchisq(urvx2,1)
     x2 = (x2 - mdchmean)/mdchsd
     z = rnorm(nobs,0,1)
     oddrat = exp(trubet[1] + trubet[2]*x1 + trubet[3]*x2 + trubet[4]*z)
     pr = oddrat/(1+oddrat)
     y = rbinom(nobs,1,pr)
     urve1 = 0.9746527*runif(nobs)
     e1 = qchisq(urve1,1)
     e1 = (e1 - mdchmean)/mdchsd
     e1 = sig*e1
     w1 = x1 + e1
     urve2 = 0.9746527*runif(nobs)
     e2 = qchisq(urve2,1)
     e2 = (e2 - mdchmean)/mdchsd
     e2 = sig*e2
     w2 = x2 + e2
    }

# GENERATE ZETA'S
zeta1 = rnorm(n,0,1)
zeta2 = rnorm(n,0,1)
for (kk in 2:k) {
  zz1 = rnorm(n,0,1)
  zeta1 = cbind(zeta1,zz1)
  zz2 = rnorm(n,0,1)
  zeta2 = cbind(zeta2,zz2)
 }
szeta1 = sig1*zeta1
szeta2 = sig2*zeta2 

vare <- sig1^2

# NAIVE ESTIMATE
naive = glm(y~w1+w2+z,family=binomial)
betnai = c(naive$coef[1],naive$coef[2],naive$coef[3],naive$coef[4])
simex1 <- simex(model = naive , SIMEXvariable = c("w1","w2"), B=200,  lambda=c(0.5,0.7,0.9,1.1,1.3,1.5,1.7,1.9) , measurement.error = c(sqrt(vare),sqrt(vare)), 
fitting.method = "quadratic", jackknife.estimation =
"quadratic" , asymptotic = FALSE)
simex2 <- refit(simex1,"linear",jackknife.estimation = "linear" ,asymptotic=FALSE)
simex3 <- try(refit(simex1,"nonlinear",jackknife.estimation = "nonlinear" ,asymptotic=FALSE),TRUE)

#print(simex1$variance.jackknife)
#print(simex2$variance.jackknife)
#print(simex3$variance.jackknife)

# COMPUTE CORRECTED SCORE ESTIMATOR
dim(w1) = c(n,1)
dim(w2) = c(n,1)
dim(z) = c(n,1)
beta = betnai      
cntl = list(1e-6,1e-6,1e-3,100)
names(cntl) = c('xtol','ftol','btol','maxit')
funest = nleqslv(beta,corscor,jac=NULL,y=y,w1=w1,w2=w2,z=z,szeta1=szeta1,szeta2=szeta2,control=cntl)
bet0 = funest$x[1]
bet1 = funest$x[2]
bet2 = funest$x[3]
bet3 = funest$x[4]

#store estimates
#row = parameter, column = method

for (parindx in 1:4) {
  estimates[i,parindx,1] = betnai[parindx]
  estimates[i,parindx,2] = ifelse(abs(funest$x[parindx])<5,funest$x[parindx],NA)
  estimates[i,parindx,3] = ifelse(abs(simex1$coef[parindx])<5,simex1$coef[parindx],NA)
  estimates[i,parindx,4] = ifelse(abs(simex2$coef[parindx])<5,simex2$coef[parindx],NA)
}

if (inherits(simex3,"try-error")==FALSE) { 
  for (parindx in 1:4) {
    estimates[i,parindx,5] = simex3$coef[parindx]
    estimates[i,parindx,5] = ifelse(abs(estimates[i,parindx,5])<5,estimates[i,parindx,5],NA)
    sds[i,parindx,5] = sqrt(simex3$variance.jackknife[parindx,parindx])
    sds[i,parindx,5] = ifelse(abs(estimates[i,parindx,5])<5,sds[i,parindx,5],NA)
    sds[i,parindx,5] = ifelse(sds[i,parindx,5]>0,sds[i,parindx,5],NA)
  }
}
else {
  for (parindx in 1:4) {
    estimates[i,parindx,5] = NA
    sds[i,parindx,5] = NA
  }
}

for (rr in 1:4) {
    sds[i,rr,3] = sqrt(simex1$variance.jackknife[rr,rr])
    sds[i,rr,3] = ifelse(abs(estimates[i,rr,3])<5,sds[i,rr,3],NA)
    sds[i,rr,4] = sqrt(simex2$variance.jackknife[rr,rr])
    sds[i,rr,4] = ifelse(abs(estimates[i,rr,4])<5,sds[i,rr,4],NA)
}

trubet <- c(trub0,trub1,trub2,trub3)
for (rr in 1:4) {
  for (sm in 3:5) {
    diff = abs(estimates[i,rr,sm]-trubet[rr]) - 1.96*sds[i,rr,sm]
    cicov[i,rr,sm] = (diff <= 0)
#print(c(i,rr,sm,diff,cicov[i,rr,sm]))
  }
}


}

#SUMMARY STATISTICS
for (imeth in 1:5) {
for (pp in 1:4) {
  estvec = estimates[,pp,imeth]
  sdvec = sds[,pp,imeth]
  civec = cicov[,pp,imeth]
  estmat[pp,imeth] = mean(estvec,na.rm=TRUE)
  medmat[pp,imeth] = median(estvec,na.rm=TRUE)
  empsdmat[pp,imeth] = sd(estvec,na.rm=TRUE)
  iqrmat[pp,imeth]<-(summary(estvec)[5]-summary(estvec)[2])/1.349
  cimat[pp,imeth] = mean(civec,na.rm=TRUE)
  sdmat[pp,imeth] = mean(sdvec,na.rm=TRUE)
  nacount[pp,imeth] =  sum(is.na(sdvec))
#print(summary(estvec))
 }
 }

#PRINT OUT RESULTS: naive, n-s, simex1-3
output<-cbind(t(estmat)[,1],t(medmat)[,1],t(empsdmat)[,1],t(iqrmat)[,1],t(estmat)[,2],t(medmat)[,2],t(empsdmat)[,2],t(iqrmat)[,2])
print(output,digits=getOption("digits"))
output<-cbind(t(estmat)[,3],t(medmat)[,3],t(empsdmat)[,3],t(iqrmat)[,3],t(estmat)[,4],t(medmat)[,4],t(empsdmat)[,4],t(iqrmat)[,4])
print(output,digits=getOption("digits"))


#PRINT OUT RESULTS 
ee = data.frame(estmat)
ss = data.frame(empsdmat)
ci = data.frame(cimat)
ss1 = data.frame(sdmat)
rownames = rbind(c("Intercept"),c("Slope1"),c("Slope2"),c("Slope3"))
ee = cbind(rownames,ee)
ss = cbind(rownames,ss)
ss1 = cbind(rownames,ss1)
ci = cbind(rownames,ci)
colname = c("parameter","naive","mc-ce","simex-q","simex-l","simex-nl")
names(ee) = colname
names(ss) = colname
names(ci) = colname 
names(ss1) = colname
print("Mean of Estimates")
print(ee)
print("Empirical SD of Estimates")
print(ss)   
print("Mean Estimated SD")
print(ss1)
print("95 coverage rates")
print(ci) 
print("Number of NAs in CI")
print(nacount)

